@@ -16,7 +16,10 @@ bool WideToChar(const wchar *Src,char *Dest,int DestSize)
 #else
 #ifdef MBFUNCTIONS
 
-  if (wcstombs(Dest,Src,DestSize)==(size_t)-1)
+  size_t ResultingSize=wcstombs(Dest,Src,DestSize);
+  if (ResultingSize==(size_t)-1)
+    RetCode=false;
+  if (ResultingSize==0 && *Src!=0)
     RetCode=false;
 
   if ((!RetCode || *Dest==0 && *Src!=0) && DestSize>NM && strlenw(Src)<NM)
@@ -66,7 +69,10 @@ bool CharToWide(const char *Src,wchar *Dest,int DestSize)
 #else
 #ifdef MBFUNCTIONS
 
-  if (mbstowcs(Dest,Src,DestSize)==(size_t)-1)
+  size_t ResultingSize=mbstowcs(Dest,Src,DestSize);
+  if (ResultingSize==(size_t)-1)
+    RetCode=false;
+  if (ResultingSize==0 && *Src!=0)
     RetCode=false;
 
   if ((!RetCode || *Dest==0 && *Src!=0) && DestSize>NM && strlen(Src)<NM)
@@ -241,11 +247,11 @@ wchar* strcpyw(wchar *dest,const wchar *src)
 }
 
 
-wchar* strncpyw(wchar *dest,const wchar *src,int n)
+wchar* strncpyw(wchar *dest,const wchar *src,size_t n)
 {
   do {
     *(dest++)=*src;
-  } while (*(src++)!=0 && --n > 0);
+  } while (*(src++)!=0 && (int)(--n) > 0);
   return(dest);
 }
 
@@ -257,11 +263,11 @@ wchar* strcatw(wchar *dest,const wchar *src)
 
 
 #ifndef SFX_MODULE
-wchar* strncatw(wchar *dest,const wchar *src,int n)
+wchar* strncatw(wchar *dest,const wchar *src,size_t n)
 {
   dest+=strlenw(dest);
   while (true)
-    if (--n<0)
+    if ((int)(--n)<0)
     {
       *dest=0;
       break;
@@ -287,9 +293,9 @@ int strcmpw(const wchar *s1,const wchar *s2)
 }
 
 
-int strncmpw(const wchar *s1,const wchar *s2,int n)
+int strncmpw(const wchar *s1,const wchar *s2,size_t n)
 {
-  while (n-->0)
+  while ((int)(n--)>0)
   {
     if (*s1<*s2)
       return(-1);
@@ -316,7 +322,7 @@ int stricmpw(const wchar *s1,const wchar *s2)
 
 
 #if !defined(SFX_MODULE) && !defined(_WIN_CE)
-inline int strnicmpw_w2c(const wchar *s1,const wchar *s2,int n)
+inline int strnicmpw_w2c(const wchar *s1,const wchar *s2,size_t n)
 {
   wchar Wide1[NM*2],Wide2[NM*2];
   strncpyw(Wide1,s1,sizeof(Wide1)/sizeof(Wide1[0])-1);
@@ -332,7 +338,7 @@ inline int strnicmpw_w2c(const wchar *s1,const wchar *s2,int n)
 
 
 #ifndef SFX_MODULE
-int strnicmpw(const wchar *s1,const wchar *s2,int n)
+int strnicmpw(const wchar *s1,const wchar *s2,size_t n)
 {
   return(strnicmpw_w2c(s1,s2,n));
 }
@@ -394,6 +400,20 @@ wchar* strupperw(wchar *Str)
 #endif
 
 
+#ifndef SFX_MODULE
+wchar* strdupw(const wchar *Str)
+{
+  if (Str==NULL)
+    return(NULL);
+  wchar *n=(wchar *)malloc((strlenw(Str)+1)*sizeof(wchar));
+  if (n==NULL)
+    return(NULL);
+  strcpyw(n,Str);
+  return(n);
+}
+#endif
+
+
 #ifndef SFX_MODULE
 int toupperw(int ch)
 {
